from typing import Union, List, Dict


class DeviceInfo:
    def __init__(self,
                 name: str,
                 pretty_name: str,
                 default: bool = False,
                 availability_func: callable = lambda: True):
        self.name = name
        self.pretty_name = pretty_name
        self.default = default
        self.availability_func = availability_func
        self.availability = None # cached value

    def is_available(self):
        if self.availability is None:
            self.availability = self.availability_func()
        return self.availability

    def has_availability(self):
        return self.availability is not None

class DeviceManager:
    def __init__(self):
        # Full info about all operation modes
        self.device_list = {
            "gpu": DeviceInfo(
                name="gpu",
                pretty_name="GPU",
                default=True,
                availability_func=lambda: __import__('torch').cuda.is_available()
            ),
            "cpu": DeviceInfo(
                name="cpu",
                pretty_name="CPU",
                default=False
            ),
            "cpu_max": DeviceInfo(
                name="cpu_max",
                pretty_name="CPU (max speed)",
                default=False
            ),
            "mps": DeviceInfo(
                name="mps",
                pretty_name="MPS (MacOS)",
                default=False,
                availability_func=lambda: __import__('torch').backends.mps.is_available()
            )
        }

    #Helper function to get the pieces of the mode info
    def get_all(self) -> List[str]:
        return list(self.device_list.keys())

    def get_available(self) -> List[str]:
        return [device.name for device in self.device_list.values() if device.is_available()]
        #return self.get_all()

    #Only the first occurrence of default tag will be returned
    def get_default(self, only_available = False) -> str:
        device_list = self.get_available() if only_available else self.get_all()
        for device in device_list:
            if self.device_list[device].default:
                return device
        return "cpu" # fallback

    def get_pretty_name(self, device: str) -> str:
        return self.device_list[device].pretty_name

    def is_available(self, device: str):
        return self.device_list[device].is_available()

    def has_availability(self, device: str):
        return self.device_list[device].has_availability()


from typing import Union, List, Dict

class ModeInfo:
    def __init__(self,
                 name: str,
                 pretty_name: str,
                 folds: List[Union[int, str]],
                 model_plans: List[str],
                 model_config: str,
                 suffixes: Dict[str, str],
                 archive_names: List[str],
                 default = False):
        self.name = name
        self.pretty_name = pretty_name
        self.folds = folds
        self.suffixes = suffixes
        self.model_plans = model_plans
        self.model_config = model_config
        self.archive_names = archive_names
        self.default = default

# Full info about all operation modes
mode_list = {
    "petct": ModeInfo(
        name="petct",
        pretty_name="PET/CT",
        folds=[0, 1, 2, 3, 4],
        model_plans=["nnUNetPlans", "nnUNetResEncUNetMPlans", "nnUNetResEncUNetLPlans"],
        model_config="3d_fullres",
        suffixes={"CT": "_0000",
                  "PET": "_0001"},
        archive_names=["LyROI_Orig.zip", "LyROI_ResM.zip", "LyROI_ResL.zip"],
        default=True
    )
}

#Helper function to get the pieces of the mode info
def get_mode_list() -> List[str]:
    return list(mode_list.keys())

#Only the first occurrence of default tag will be returned
def get_default_mode() -> str:
    for mode in mode_list:
        if mode_list[mode].default:
            return mode
    return ""


def get_pretty_name(mode: str) -> str:
    return mode_list[mode].pretty_name

def get_model_folders(mode: str) -> List[str]:
    from nnunetv2.utilities.file_path_utilities import get_output_folder

    mode_info = mode_list[mode]
    folder_list = [get_output_folder(1, 'nnUNetTrainer', plan, mode_info.model_config) for plan in mode_info.model_plans]
    return folder_list

def get_folds(mode: str) -> List[Union[int, str]]:
    return mode_list[mode].folds

def get_suffixes(mode: str) -> List[str]:
    return list(mode_list[mode].suffixes.values())

def get_suffix_dict(mode: str) -> Dict[str, str]:
    return mode_list[mode].suffixes

def get_archive_names(mode: str):
    return mode_list[mode].archive_names